mod page;
mod pager;
mod row;
mod statement;
mod table;

use crate::statement::ExecuteResult;
use crate::table::Table;
use std::io;
use std::io::Write;
use std::process::exit;

struct InputBuffer {
    buffer: Option<String>,
    table: Table,
}

impl InputBuffer {
    pub fn new() -> Self {
        return Self {
            buffer: None,
            table: Table::new(),
        };
    }

    pub(crate) fn print_prompt(&mut self) -> &mut Self {
        print!("db > ");
        if let Err(_) = io::stdout().flush() {
            panic!("could not print to stdout")
        }
        self
    }

    pub(crate) fn read_input(&mut self) -> Result<&mut Self, String> {
        self.buffer = Some(String::new());
        match &mut self.buffer {
            Some(input_buffer) => {
                if let Ok(_) = io::stdin().read_line(input_buffer) {
                    Ok(self)
                } else {
                    Err(String::from("could not handle input"))
                }
            }
            _ => Err(String::from("Could not initialize buffer")),
        }
    }

    pub(crate) fn parse(&mut self) {
        match &self.buffer {
            Some(command) => {
                if command.starts_with(".") {
                    Self::handle_meta_statement(command);
                } else {
                    match Self::prepare_statement(&command.replace("\n", "")) {
                        Ok(statement) => {
                            let execution_result = statement.execute(&mut self.table);
                            match execution_result {
                                ExecuteResult::Success => {
                                    println!("success")
                                }
                                _ => {
                                    println!("failure")
                                }
                            }
                        }
                        Err(e) => {
                            println!("{}", e);
                        }
                    }
                }
            }
            None => {}
        }
    }

    fn prepare_statement(command: &String) -> Result<statement::Statement, String> {
        return statement::Statement::parse_statement(command);
    }

    fn handle_meta_statement(command: &String) {
        match command.replace("\n", "").trim() {
            ".exit" => {
                exit(0);
            }
            cmd => {
                println!("Could not handle command: {cmd}")
            }
        }
    }
}

fn main() -> Result<(), String> {
    let mut input_buffer = InputBuffer::new();

    loop {
        input_buffer.print_prompt().read_input()?.parse()
    }
}
